/**
 * \file timer.c
 * \brief  
 * \date 18 . 2015 .
 * \author: ARV
 */
 
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include "config.h"
#include "timer.h"
#include "led.h"

//extern config_t config;

///      
static volatile uint16_t repeat_delay;

///     
static volatile uint16_t wait_conversion;

///      powerdown
static volatile uint16_t wait_powerdown;

static uint8_t police_blink_delay[5]={2,2,2,2,2};	//      

static uint8_t array_pos=0;
static uint8_t curent_array_delay=0;


ISR(TIMER2_OVF_vect){
	//   2
	// 8000000/1024/255=30
}

ISR(TIMER1_OVF_vect){
//    1,        (1)
	TCNT1=TIMER1_PRESCALE;

	if(repeat_delay) repeat_delay--;
	if(wait_conversion) wait_conversion--;
	if(wait_powerdown) wait_powerdown--;
}

/**  2         
 * @param 
 * @return 
 */
void timer2_init(void){
	//  2
	TCNT2 = 0;
	//TCCR2 = 0;											/// Normal port operation, OC2A/OC2B disconnected, Normal Mode
	TCCR2 = _BV(CS20) | _BV(CS21) | _BV(CS22);				/// clkI/1024
	//ASSR |=_BV(AS2);										/// Asynchronous Timer/Counter2 for powerdown
	TIMSK |= _BV(TOIE2);									/// Timer/Counter2 Overflow Interrupt Enable
	//while (ASSR & (1<<TCN2UB | 1<<OCR2UB | TCR2UB)){}		//       
	//   ,    (     )
	//          
	array_pos=0;
	curent_array_delay=police_blink_delay[array_pos];
}

/**  2    powerdown
 * @param 
 * @return 
 */
void timer2_stop(void){
	//  2
	TCNT2 = 0;
	TCCR2 = 0;												/// Normal port operation, OC2A/OC2B disconnected, Normal Mode
	TIMSK &= ~_BV(TOIE2);									/// Timer/Counter2 Overflow Interrupt Disable
	//while (ASSR & (1<<TCN2UB | 1<<OCR2UB | TCR2UB)){}		//       
}

/**  2          powerdown
 * @param measuring_time        
 * @return 
 */
void timer1_init(void){
	//   16-bit Timer/Counter1
	TCNT1 = TIMER1_PRESCALE;
	TCCR1A = 0;													/// Normal port operation, OC1A/OC1B disconnected, Normal Mode
	TCCR1B |= _BV(CS12) | _BV(CS10);							/// clkI/O/1024 (From prescaler)
	TIMSK |= _BV(TOIE1);										/// Timer/Counter1, Overflow Interrupt Enable

	reset_timer(DELAY_AUTOSEND);
	reset_timer(DELAY_REPEAT);
	reset_timer(DELAY_POWERDONW);
}

uint8_t time_is_over(delay_t t){
	switch(t){
	case DELAY_REPEAT:							///    
		ATOMIC_BLOCK(ATOMIC_RESTORESTATE){		///       
			return repeat_delay == 0;			///  1 -  repeat_delay=0,   0.
		}
		break;
	case DELAY_AUTOSEND:						///    
		ATOMIC_BLOCK(ATOMIC_RESTORESTATE){		///       
			return wait_conversion == 0;		///  1 -  wait_conversion=0,   0.
		}
		break;
	case DELAY_POWERDONW:						///     powerdown
		ATOMIC_BLOCK(ATOMIC_RESTORESTATE){		///       
			return wait_powerdown == 0;		///  1 -  wait_conversion=0,   0.
		}
		break;
	}
return 0;
}

void reset_timer(delay_t t){
	switch(t){
	case DELAY_REPEAT:
		ATOMIC_BLOCK(ATOMIC_RESTORESTATE){
			repeat_delay = 5;
		}
		break;
	case DELAY_AUTOSEND:
		ATOMIC_BLOCK(ATOMIC_RESTORESTATE){
			wait_conversion = config.delay_termo;
		}
		break;
	case DELAY_POWERDONW:
		ATOMIC_BLOCK(ATOMIC_RESTORESTATE){
			wait_powerdown = 5;
		}
		break;
	}
}
